PVector[] p = {new PVector(400, 100), new PVector(70, 90), new PVector(300, 400)}; //Locations of the stations
//STATIONS SHOULD NOT HAVE THE SAME X OR THE SAME Y COORDINATE
int t = 0;
PVector res;

import processing.serial.*;

String buff = "";
int NEWLINE = 10;
char stations[] = {'A','B','C'};
float freqs[] = new float[3];
float amps[] = new float[3];
int n;
Serial port;

PImage img_kagu;
PImage img_toucan;
PImage img_manakin;
PImage img_kingfisher;

void setup() {
  size(500,500);
  //Get the serial port of the arduino for communication
  println("Available serial ports:");
  println(Serial.list());
  port = new Serial(this, Serial.list()[1], 115200);
  img_kagu = loadImage("KAGU.PNG");
  img_toucan = loadImage("TOUCAN.PNG");
  img_manakin = loadImage("MANAKIN.PNG");
  img_kingfisher = loadImage("KINGFISHER.PNG");
}
void draw() {
  while (port.available() > 0) {
    serialEvent(port.read()); // read data
  }
  
  background(#FFFFFF);
  //noFill();
  //stroke(#000000);
  
  fill(#000000);
  textAlign(CENTER);
  textSize(20);
  for (int i = 0; i < p.length; ++i) {
    //circle(p[i].x,p[i].y,3); //draw a location for each station
    text(stations[i],p[i].x,p[i].y + 10);
  }
  noFill();
  
  if (max(amps) > 0) { //Check if we have an amplitude from every station, because otherwise we cannot calculate a location
    float v[] = new float[3];
    for (int i = 0; i < p.length; ++i) {
      v[i] = pow(amps[i], -2); //Take the amplitude and do it to the power of -2, because that should be an indication of the distance
    }
    
    //Find the station with the highest distance (biggest circle) and store it in mi
    //Change the distance values to a multiplier of the maximum distance
    float m = max(v);
    int mi = 0;
    for (int i = 0; i < p.length; ++i) {
      if (v[i] == m) mi = i;
      v[i] = v[i]/m;
      //println(stations[i] + ":" + v[i]*200);
    }
    m = 200; //Set the radius of the biggest circle to 200
    
    //Create variables for the x and y coordinate of the location
    float x = 0;
    float y = 0; //<>//
    for (int i = 0; i < 50; ++i) { // Do maximum 50 itarations in approaching the right location of the sound
      float a1 = 0;
      float b1 = 0;
      float a2 = 0;
      float b2 = 0;
      //Go through every possible pair of stations, calculate a line between the stations, and calculate a line perpendiculat to this line going through the the intersection of the circles of both stations
      for (int i1 = 0; i1 < p.length; ++i1) { 
        int i2 = (i1 + 1) % 3;
        float x1 = p[i1].x;
        float y1 = p[i1].y;
        float v1 = v[i1]*m;//Multiply the relative distance to the radius of the biggest circle that was set (this gets updated every cycle)
        float x2 = p[i2].x;
        float y2 = p[i2].y;
        float v2 = v[i2]*m;//Multiply the relative distance to the radius of the biggest circle that was set (this gets updated every cycle)
        
        float d = p[i1].dist(p[i2]);
        if (d == 0 || !Float.isFinite(d)) println("d:"+d);
        float a = (sq(v1) - sq(v2) + sq(d)) / (2*d);
        if (d == 0 || !Float.isFinite(d)) println("a:"+d);
        float xc = x1 + (x2 - x1)/d*a;
        float yc = y1 + (y2 - y1)/d*a;
        
        //Store the previously found line as line 2 instead of 1
        a2 = a1;
        b2 = b1;
        //Calculate a line through the intersections of the circle in the format y = ax+b
        a1 = (x2 - x1)/(y1 - y2);
        b1 = yc - (a1*xc);
      }
      // Calculate the intersection of the lines with the following formula
      x = (b2 - b1)/(a1 - a2);
      y = a1 * x + b1;
      res = new PVector(x,y); //Create a vector with the estimated location
      float mnew = p[mi].dist(res); //Calculate the new size of the biggest circle
      if (abs(m - mnew) < 1) break; //If the difference between the new estimation and the previous estimation is less than 1 pixel, stop with making more estimations
      m = mnew; //Set the size of the biggest circle to the new size.
    }
    circle(x,y,3); //Draw a circl on the final estimated location
    

    
    
    //Draw circles from the stations to this point.
    stroke(#0000FF);
    for (int i = 0; i < p.length; ++i) {
      circle(p[i].x,p[i].y,p[i].dist(res)*2);
    }
    
    //Get frequency and compare to birds, show pic of relevant bird
    float avgfreq = (freqs[0] + freqs[1] + freqs[2])/3;
    if (avgfreq > 900 && avgfreq < 1200) {
      image(img_kagu,x-20,y-20,40,40);
    } else if (avgfreq < 400 && avgfreq > 600) {
      image(img_toucan,x-20,y-20,40,40);
    } else if (avgfreq > 1900 && avgfreq < 2200) {
      image(img_manakin,x-20,y-20,40,40);
    } else if (avgfreq > 2700 && avgfreq < 3500) {
      image(img_kingfisher,x-20,y-20,40,40);
    }
  }
}

/*
Transmission protocol:

AF10A20
BF43A21
CF98A12

-> Sensor A has a frequency of 10 and an amplitude of 20, Sensor B has a Frequncy of 43 and an amplitude of 21, etc.
*/
String valid = "ABCF0123456789.\r\n";
void serialEvent(int serial) 
{ 
  try  {    // try-catch because of transmission errors
    if(serial != NEWLINE) { 
      buff += char(serial);
      //println(valid.indexOf(char(serial)));
      if (valid.indexOf(char(serial)) == -1) buff = ""; //Empty buffer when encountering an invalid character
    } else {
      println(buff);
      buff = buff.substring(0, buff.length()-1);// Discard the carriage return at the end of the buffer
      char station = buff.charAt(0); // The first character tells us which station the value is from
      buff = buff.substring(1); // Remove it from the string
      if (buff.charAt(0) != 'F') throw new Exception("Second character is not F");
      buff = buff.substring(1); // Remove it from the string
      String[] buff_split = split(buff,"A"); //Get the number before A (Frequency) and after (Amplitude)
      
      // Parse the String into two floats
      for(int z=0;z<3;z++) {
        if(station == stations[z]) {
          freqs[z] = Float.parseFloat(buff_split[0]); //Use the first number as frequency
          amps[z] = Float.parseFloat(buff_split[1]); //Use the second number as amplitude
        }
      }
      buff = ""; // Clear the value of "buff"
    }
  }
  catch(Exception e) {
    println("no valid data:" + e);
    buff = ""; // Clear the value of "buff"
  }
}
